# -*- coding: utf-8 -*-
import requests
import certifi
import os

class SandCage:
    API_ROOT = 'https://api.sandcage.com/'
    API_VERSION = '0.2'
    TIMEOUT = 30
    TRUSTED = certifi.where()
    
    def __init__(self, api_key=None):
        """Give your SandCage API Key as a constructor parameter
        This can be retrieved from https://www.sandcage.com/panel/api_key
        """
        
        self.api_key = api_key
        self.sandcage_api_endpoint = '{0}/{1}/'.format(self.API_ROOT,
                                                       self.API_VERSION)
        self.user_agent = '{0} - {1}'.format('SandCage', self.API_VERSION)

    def _get_post_data(self, payload=None):
        """Combine payload with API KEY and return the dictionary."""
        
        d = {'key': self.api_key}
        if payload:
            d.update(payload)
        return d

    def _update_callback(self, data, callback_endpoint=None):
        """Update data with callback_endpoint address."""
        
        if callback_endpoint:
            data.update({'callback_url': callback_endpoint})

    def call_endpoint(self, endpoint, post=False, data=None):
        """The call_endpoint method

        * This method sends http request to the endpoint

        * First parameter "endpoint" (string) is the url of the endpoint.
        * Second parameter "post" (boolean) indicates the http method type.
          True => POST
          False => GET
        * Third parameter "data" (dictionary) is the data to be sent.
        * Method's return type is Response object.
        """
        
        headers = {'User-Agent': self.user_agent}
        if post:
            r = requests.post(endpoint, headers=headers, timeout=self.TIMEOUT,
                              json=data, verify=self.TRUSTED)
        else:
            r = requests.get(endpoint, headers=headers, timeout=self.TIMEOUT,
                             verify=self.TRUSTED)
        return r

    def get_info_service(self, payload=None):
        """The "get-info" service
        
        * First parameter "payload" (dictionary) contains the data to be sent.
        * Method's return type is Response object.
        """
        return self.call_endpoint(self.sandcage_api_endpoint + 'get-info', post=True,
                         data=self._get_post_data(payload))

    def list_files_service(self, payload=None):
        """The "list-files" service
        
        * First parameter "payload" (dictionary) contains the data to be sent.
        * Method's return type is Response object.
        """
        return self.call_endpoint(self.sandcage_api_endpoint + 'list-files', post=True,
                         data=self._get_post_data(payload))

    def destroy_files_service(self, payload=None, callback_endpoint=None):
        """The "destroy-files" service
        
        * First parameter "payload" (dictionary) contains the data to be sent.
        * Second parameter "callback_endpoint" (string) is the url, where the 
          callback should be send to.
        * Method's return type is Response object.
        """

        data = self._get_post_data(payload)
        self._update_callback(data, callback_endpoint)
        return self.call_endpoint(self.sandcage_api_endpoint + 'destroy-files', post=True,
                         data=data)

    def schedule_files_service(self, payload=None, callback_endpoint=None):
        """The "schedule-tasks" service

        * First parameter "payload" (dictionary) contains the data to be sent.
        * Second parameter "callback_endpoint" (string) is the url, where the 
          callback should be send to.
        * Method's return type is Response object.
        """
         
        data = self._get_post_data(payload)
        self._update_callback(data, callback_endpoint)
        return self.call_endpoint(self.sandcage_api_endpoint + 'schedule-tasks',
                         post=True, data=data)




"""A setuptools based setup module.

See:
https://packaging.python.org/en/latest/distributing.html
"""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='sandcage',

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version='0.2.0',

    description='Python client library for SandCage\'s API',
    long_description=long_description,

    # The project's main homepage.
    url='https://github.com/sandcage/sandcage-api-python',

    # Author details
    author='Antti Nykanen <ajjn@iki.fi>, Evangelos Kokkonos',
    author_email='ajjn@iki.fi',

    # Choose your license
    license='Apache License Version 2.0',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 4 - Beta',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: Apache Software License',
        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],

    # What does your project relate to?
    keywords='api http development',

    packages = find_packages('src'),
    package_dir = {'': 'src'},
    py_modules = ['sandcage'],
    include_package_data=True,
    zip_safe=False,
    
    # List run-time dependencies here.  These will be installed by pip when
    # your project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=['requests',
                      'certifi'],

)
